//Mugurel Ionut Andreica - Backtracking
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define PRINT_BKT_SOL 0

#define MOD 1000000007

#define FILEIN "dsets.in"
#define FILEOUT "dsets.out"

long long D;
int DP;

void ReadInput() {
	freopen(FILEIN, "r", stdin);
	scanf("%lld", &D);
	DP = D & 1;
}

long long answer;

// Backtracking solution.

#define NMAX 128
char A[NMAX][NMAX];

int bkt_answer, nsel, dmax, d, col_zero;

void BKT(int i, int j) {
	if (i == D + 1) {
		if (dmax == D && col_zero) {
			bkt_answer++;
			if (PRINT_BKT_SOL) {
				fprintf(stderr, "Bkt sol %u:\n", bkt_answer);
				for (i = 0; i <= D; i++) {
					for (j = 0; j <= D; j++)
						fprintf(stderr, "%d", A[i][j]);
					fprintf(stderr, "\n");
				}
				fprintf(stderr, "\n");
			}
		}
		return;
	}

	if (j > D) {
		if (nsel == 0)
			return;
		BKT(i + 1, 0);
		return;
	}

	A[i][j] = 1;
	nsel++;

	// Compute the new maximum distance.
	int p, q, old_dmax = dmax, old_col_zero = col_zero;

	if (j == 0)
		col_zero = 1;

	for (p = 0; p <= i; p++)
		for (q = 0; q <= D; q++)
			if (A[p][q]) {
				d = i - p;
				if (q <= j)
					d += (j - q);
				else
					d += (q - j);
				if (d > dmax)
					dmax = d;
			}

	if (dmax <= D)
		BKT(i, j + 1);

	A[i][j] = 0;
	nsel--;
	dmax = old_dmax;
	col_zero = old_col_zero;
	BKT(i, j + 1);
}

void SolveBKT() {
	bkt_answer = nsel = dmax = col_zero = 0;
	BKT(0, 0);
	answer = (long long) bkt_answer;
}

// O(D) solution.

#define DMAX 500001
#define BMAX 10

long long bit[DMAX];
int small_bit[BMAX];

void ComputeBits() {
	int i;

	bit[0] = 1;
	for (i = 1; i < DMAX; i++)
		bit[i] = (bit[i - 1] * 2) % MOD;

	small_bit[0] = 1;
	for (i = 1; i < BMAX; i++)
		small_bit[i] = small_bit[i - 1] << 1;
}

int cleft[2], cright[2], cup[2], cdown[2], ncleft, ncright, ncup, ncdown, nc;

void SolveLinear() {
	int C, CP, CPUP, CPDOWN;
	int lup, ldown, lleft, lright, conf, i, any_corner;
	long long ntotal, nin, ninold, tmp, tmpans, bitninold, bitnin;

	ComputeBits();

	answer = ninold = 0;
	bitninold = 1;

	for (C = DP; C <= D; C++) {
		if (C == 0) {
			// Special case.
			answer = (answer + bit[(D / 2)]) % MOD;
			continue;
		}

		CP = C & 1;

		for (CPUP = 0; CPUP <= 1; CPUP++) {
			CPDOWN = CPUP ^ CP;

			if (DP == 0) {
				if (CPUP == 0) {
					lup = (D / 2) + 1;
					lleft = lright = (C / 2) + 1;
				} else {
					lup = D / 2;
					lleft = lright = (C + 1) / 2;
				}

				if (CPDOWN == 0)
					ldown = (D / 2) + 1;
				else
					ldown = D / 2;

				ntotal = (long long) lleft * (long long) ((D / 2) + 1) +
						(long long) (C + 1 - lleft) * (long long) (D / 2);
			} else {
				lup = ldown = (D + 1) / 2;
				if (CPUP == 0) {
					lleft = (C / 2) + 1;
					lright = (C + 1) / 2;
				} else {
					lleft = (C + 1) / 2;
					lright = (C / 2) + 1;
				}

				ntotal = (long long) (C + 1) * (long long) ((D + 1) / 2);
			}

			nc = ncleft = ncright = ncup = ncdown = 0;
			if (CPUP == 0) {
				cleft[ncleft] = nc;
				ncleft++;
				cup[ncup] = nc;
				ncup++;
				nc++;
			}

			if (CPUP == DP) {
				cup[ncup] = nc;
				ncup++;
				cright[ncright] = nc;
				ncright++;
				nc++;
			}

			if (DP == CPDOWN) {
				cright[ncright] = nc;
				ncright++;
				cdown[ncdown] = nc;
				ncdown++;
				nc++;
			}

			if (CPDOWN == 0) {
				cdown[ncdown] = nc;
				ncdown++;
				cleft[ncleft] = nc;
				ncleft++;
				nc++;
			}

			nin = ntotal - (lleft + lup + lright + ldown) + nc;

			if (nin - ninold < 0 || nin - ninold >= DMAX) {
				fprintf(stderr, "bad nin dif=%lld\n", nin - ninold);
				exit(1);
			}

			bitnin = (bitninold * bit[nin - ninold]) % MOD;

			tmpans = 0;
			for (conf = 0; conf < (1 << nc); conf++) {
				tmp = 1;

				// Handle the left side.
				for (i = any_corner = 0; i < ncleft; i++)
					if ((conf & small_bit[cleft[i]]) > 0) {
						any_corner = 1;
						break;
					}

				if (!any_corner && lleft - ncleft == 0)
					continue;
				if (any_corner)
					tmp = (tmp * bit[lleft - ncleft]) % MOD;
				else
					tmp = (tmp * (bit[lleft - ncleft] - 1 + MOD)) % MOD;

				// Handle the up side.
				for (i = any_corner = 0; i < ncup; i++)
					if ((conf & small_bit[cup[i]]) > 0) {
						any_corner = 1;
						break;
					}
				if (!any_corner && lup - ncup == 0)
					continue;
				if (any_corner)
					tmp = (tmp * bit[lup - ncup]) % MOD;
				else
					tmp = (tmp * (bit[lup - ncup] - 1 + MOD)) % MOD;

				// Handle the right side.
				for (i = any_corner = 0; i < ncright; i++)
					if ((conf & small_bit[cright[i]]) > 0) {
						any_corner = 1;
						break;
					}
				if (!any_corner && lright - ncright == 0)
					continue;
				if (any_corner)
					tmp = (tmp * bit[lright - ncright]) % MOD;
				else
					tmp = (tmp * (bit[lright - ncright] - 1 + MOD)) % MOD;

				// Handle the down side.
				for (i = any_corner = 0; i < ncdown; i++)
					if ((conf & small_bit[cdown[i]]) > 0) {
						any_corner = 1;
						break;
					}
				if (!any_corner && ldown - ncdown == 0)
					continue;
				if (any_corner)
					tmp = (tmp * bit[ldown - ncdown]) % MOD;
				else
					tmp = (tmp * (bit[ldown - ncdown] - 1 + MOD)) % MOD;

				tmp = (tmp * bitnin) % MOD;
				if (C < D)
					tmp = (tmp * 2) % MOD;
				tmpans = (tmpans + tmp) % MOD;
			}

			answer = (answer + tmpans) % MOD;

			//if (C % 10000 == 0)
			//	fprintf(stderr, "D=%d C=%d CPUP=%d DP=%d CPDOWN=%d lleft=%d lup=%d lright=%d ldown=%d nc=%d ncleft=%d ncup=%d ncright=%d ncdown=%d ntotal=%lld nin=%lld (dif=%lld) bitnin=%lld tmpans=%lld\n", D, C, CPUP, DP, CPDOWN, lleft, lup, lright, ldown, nc, ncleft, ncup, ncright, ncdown, ntotal, nin, nin - ninold, bitnin, tmpans);

			if (CPUP == 1) {
				ninold = nin;
				bitninold = bitnin;
			}
		}
	}
}

// O(log(D)) solution.

#define LLBMAX 63

long long llbit[LLBMAX];

void ComputeLLBits() {
	llbit[0] = 1;
	for (int i = 1; i < LLBMAX; i++)
		llbit[i] = llbit[i - 1] << 1;
}

long long RaiseToPower(long long x, long long y) {
	long long result = 1;
	for (int i = LLBMAX - 1; i >= 0; i--) {
		result = (result * result) % MOD;
		if ((y & llbit[i]) > 0)
			result = (result * x) % MOD;
	}
	return result;
}

void SolveForC(long long C, long long& result0, long long& result1) {
	long long lleft, lright, ntotal;

	result0 = result1 = 0;

	if (C == 0 && DP == 0) {
		result0 = RaiseToPower(2, D / 2 - 1);
		return;
	}

	if (C <= 0)
		return;

	if (DP == 0) {
		lleft = lright = (C / 2) + 1;
		ntotal = (long long) lleft * (long long) ((D / 2) + 1) +
				(long long) (C + 1 - lleft) * (long long) (D / 2);
		result0 = RaiseToPower(2, ntotal - lleft - lright);
		result0 = (result0 * (RaiseToPower(2, lleft) + MOD - 1)) % MOD;
		result0 = (result0 * (RaiseToPower(2, lright) + MOD - 1)) % MOD;

		lleft = lright = (C + 1) / 2;
		ntotal = (long long) lleft * (long long) ((D / 2) + 1) +
				(long long) (C + 1 - lleft) * (long long) (D / 2);
		result1 = RaiseToPower(2, ntotal - lleft - lright);
		result1 = (result1 * (RaiseToPower(2, lleft) + MOD - 1)) % MOD;
		result1 = (result1 * (RaiseToPower(2, lright) + MOD - 1)) % MOD;
	} else {
		lleft = (C / 2) + 1;
		lright = (C + 1) / 2;
		ntotal = (long long) (C + 1) * (long long) ((D + 1) / 2);
		result0 = RaiseToPower(2, ntotal - lleft - lright);
		result0 = (result0 * (RaiseToPower(2, lleft) + MOD - 1)) % MOD;
		result0 = (result0 * (RaiseToPower(2, lright) + MOD - 1)) % MOD;
		result1 = result0;
	}
}

void SolveLog() {
	ComputeLLBits();

	long long D0, D1, DM10, DM11, DM20, DM21;

	SolveForC(D, D0, D1);
	SolveForC(D - 1, DM10, DM11);
	SolveForC(D - 2, DM20, DM21);

	//fprintf(stderr, "D=%lld: %lld %lld\n", D, D0, D1);
	//fprintf(stderr, "D-1=%lld: %lld %lld\n", D - 1, DM10, DM11);
	//fprintf(stderr, "D-2=%lld: %lld %lld\n", D - 2, DM20, DM21);

	answer = (2 * (DM10 + DM11 - DM20 - DM21 + 2 * MOD)) % MOD;
	answer = (answer + D0 - DM10 - DM11 + DM21 + 2 * MOD) % MOD;
	answer = (answer + D1 - DM11 - DM10 + DM20 + 2 * MOD) % MOD;
}

void WriteOutput() {
	freopen(FILEOUT, "w", stdout);
	printf("%lld\n", answer);
	fprintf(stderr, "D=%lld: answer=%lld\n", D, answer);
}

int main() {
	int tstart = clock();

	ReadInput();
	SolveBKT();
	//SolveLinear();
	//SolveLog();
	WriteOutput();

	fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
	return 0;
}
